@@ -22,14 +22,11 @@ use Encode 'encode';
 
 use Carp 'croak';
 
-our $VERSION = '0.0248';
+our $VERSION = '0.0249';
 
 our $COMPILER;
 our @PACKAGE_INFOS;
-our %FIELD_SYMTABLE;
 our $API;
-our @TYPE_NAMES;
-our %TYPE_SYMTABLE;
 
 sub _get_dll_file {
   my $package_name = shift;
@@ -127,17 +124,6 @@ CHECK {
   # Compile SPVM source code
   compile();
   
-  my $sub_native_address = get_sub_native_address('SPVM::std::sum_int');
-  
-  # Build type names
-  build_type_names();
-  
-  # Build type names
-  build_type_symtable();
-  
-  # Build field symbol table
-  build_field_symtable();
-  
   # Build run-time
   build_runtime();
   
@@ -150,6 +136,7 @@ CHECK {
   # Build SPVM subroutine
   build_spvm_subs();
 }
+
 sub new_string_raw {
   my $string = shift;
   
