@@ -6,16 +6,18 @@
 #include "spvm_constant_pool.h"
 #include "spvm_constant.h"
 #include "spvm_package.h"
-#include "spvm_field.h"
+#include "spvm_field_info.h"
 #include "spvm_sub.h"
 #include "spvm_util_allocator.h"
 #include "spvm_dynamic_array.h"
 #include "spvm_op.h"
 #include "spvm_constant_pool_sub.h"
-#include "spvm_constant_pool_field.h"
+#include "spvm_constant_pool_field_info.h"
 #include "spvm_constant_pool_package.h"
 #include "spvm_constant_pool_type.h"
 #include "spvm_type.h"
+#include "spvm_hash.h"
+#include "spvm_compiler.h"
 
 void SPVM_CONSTANT_POOL_adjust_alignment(SPVM_COMPILER* compiler, SPVM_CONSTANT_POOL* constant_pool, int32_t byte_size) {
   
@@ -87,41 +89,54 @@ void SPVM_CONSTANT_POOL_extend(SPVM_COMPILER* compiler, SPVM_CONSTANT_POOL* cons
 int32_t SPVM_CONSTANT_POOL_push_type(SPVM_COMPILER* compiler, SPVM_CONSTANT_POOL* constant_pool, SPVM_TYPE* type) {
   (void)compiler;
   
-  int32_t start_index = constant_pool->length;
+  int32_t id = constant_pool->length;
   
   // Extend
   int32_t extend_length = SPVM_CONSTANT_POOL_calculate_extend_length(compiler, constant_pool, sizeof(SPVM_CONSTANT_POOL_TYPE));
   SPVM_CONSTANT_POOL_extend(compiler, constant_pool, extend_length);
-
+  
   // Constant pool type information
   SPVM_CONSTANT_POOL_TYPE constant_pool_type;
+  memset(&constant_pool_type, 0, sizeof(SPVM_CONSTANT_POOL_TYPE));
   
-  constant_pool_type.id = type->id;
+  constant_pool_type.code = type->code;
+  
+  // Add length
+  constant_pool->length += extend_length;
   
   // Push type name to constant pool
   constant_pool_type.name_id = SPVM_CONSTANT_POOL_push_string(compiler, constant_pool, type->name);
   
-  constant_pool_type.element_type_id = type->element_type_id;
-  
-  if (type->op_package) {
-    constant_pool_type.package_id = type->op_package->uv.package->constant_pool_index;
+  // Parent type id
+  char* parent_type_name = SPVM_TYPE_get_parent_name(compiler, type->name);
+  SPVM_TYPE* parent_type = (SPVM_TYPE*)SPVM_HASH_search(compiler->type_symtable, parent_type_name, strlen(parent_type_name));
+  if (parent_type) {
+    constant_pool_type.parent_type_id = parent_type->id;
   }
-  else {
-    constant_pool_type.package_id = -1;
+
+  // Element type id
+  char* element_type_name = SPVM_TYPE_get_element_name(compiler, type->name);
+  if (element_type_name) {
+    SPVM_TYPE* element_type = (SPVM_TYPE*)SPVM_HASH_search(compiler->type_symtable, element_type_name, strlen(element_type_name));
+    if (element_type) {
+      constant_pool_type.element_type_id = element_type->id;
+    }
   }
-  constant_pool_type.is_array = type->is_array;
   
-  constant_pool_type.is_package = type->is_package;
+  constant_pool_type.dimension = type->dimension;
   
-  memcpy(&constant_pool->values[start_index], &constant_pool_type, sizeof(SPVM_CONSTANT_POOL_TYPE));
+  assert(type->base_type);
+  constant_pool_type.base_id = type->base_type->id;
   
-  return start_index;
+  memcpy(&constant_pool->values[id], &constant_pool_type, sizeof(SPVM_CONSTANT_POOL_TYPE));
+  
+  return id;
 }
 
 int32_t SPVM_CONSTANT_POOL_push_package(SPVM_COMPILER* compiler, SPVM_CONSTANT_POOL* constant_pool, SPVM_PACKAGE* package) {
   (void)compiler;
   
-  int32_t start_index = constant_pool->length;
+  int32_t id = constant_pool->length;
   
   // Extend
   int32_t extend_length = SPVM_CONSTANT_POOL_calculate_extend_length(compiler, constant_pool, sizeof(SPVM_CONSTANT_POOL_PACKAGE));
@@ -129,6 +144,7 @@ int32_t SPVM_CONSTANT_POOL_push_package(SPVM_COMPILER* compiler, SPVM_CONSTANT_P
 
   // Constant pool package information
   SPVM_CONSTANT_POOL_PACKAGE constant_pool_package;
+  memset(&constant_pool_package, 0, sizeof(SPVM_CONSTANT_POOL_PACKAGE));
   constant_pool_package.fields_length = package->op_fields->length;
   constant_pool_package.object_fields_length = SPVM_PACKAGE_get_object_fields_length(compiler, package);
   
@@ -138,9 +154,6 @@ int32_t SPVM_CONSTANT_POOL_push_package(SPVM_COMPILER* compiler, SPVM_CONSTANT_P
   // Push package name to constant pool
   const char* package_name = package->op_name->uv.name;
   constant_pool_package.name_id = SPVM_CONSTANT_POOL_push_string(compiler, constant_pool, package_name);
-  
-  // Type id
-  constant_pool_package.type_id = package->op_type->uv.type->id;
 
   // Push fields constant_pool indexes to constant pool
   {
@@ -148,14 +161,14 @@ int32_t SPVM_CONSTANT_POOL_push_package(SPVM_COMPILER* compiler, SPVM_CONSTANT_P
     constant_pool_package.fields_base = constant_pool->length;
     for (field_pos = 0; field_pos < package->op_fields->length; field_pos++) {
       SPVM_OP* op_field = SPVM_DYNAMIC_ARRAY_fetch(package->op_fields, field_pos);
-      SPVM_FIELD* field = op_field->uv.field;
-      SPVM_CONSTANT_POOL_push_int(compiler, constant_pool, field->constant_pool_index);
+      SPVM_FIELD_INFO* field = op_field->uv.field;
+      SPVM_CONSTANT_POOL_push_int(compiler, constant_pool, field->id);
     }
   }
   
-  memcpy(&constant_pool->values[start_index], &constant_pool_package, sizeof(SPVM_CONSTANT_POOL_PACKAGE));
+  memcpy(&constant_pool->values[id], &constant_pool_package, sizeof(SPVM_CONSTANT_POOL_PACKAGE));
   
-  return start_index;
+  return id;
 }
 
 int32_t SPVM_CONSTANT_POOL_push_sub(SPVM_COMPILER* compiler, SPVM_CONSTANT_POOL* constant_pool, SPVM_SUB* sub) {
@@ -163,7 +176,7 @@ int32_t SPVM_CONSTANT_POOL_push_sub(SPVM_COMPILER* compiler, SPVM_CONSTANT_POOL*
   
   SPVM_CONSTANT_POOL_adjust_alignment(compiler, constant_pool, sizeof(void*));
   
-  int32_t start_index = constant_pool->length;
+  int32_t id = constant_pool->length;
   
   // Extend
   int32_t extend_length = SPVM_CONSTANT_POOL_calculate_extend_length(compiler, constant_pool, sizeof(SPVM_CONSTANT_POOL_SUB));
@@ -171,6 +184,7 @@ int32_t SPVM_CONSTANT_POOL_push_sub(SPVM_COMPILER* compiler, SPVM_CONSTANT_POOL*
   
   // Set subroutine information
   SPVM_CONSTANT_POOL_SUB constant_pool_sub;
+  memset(&constant_pool_sub, 0, sizeof(SPVM_CONSTANT_POOL_SUB));
   constant_pool_sub.native_address = sub->native_address;
   constant_pool_sub.bytecode_base = sub->bytecode_base;
   constant_pool_sub.my_vars_length = sub->op_my_vars->length;
@@ -179,11 +193,21 @@ int32_t SPVM_CONSTANT_POOL_push_sub(SPVM_COMPILER* compiler, SPVM_CONSTANT_POOL*
   constant_pool_sub.is_native = sub->is_native;
   
   assert(sub->op_return_type);
+  
   constant_pool_sub.return_type_id = sub->op_return_type->uv.type->id;
   
   // Add length
   constant_pool->length += extend_length;
 
+  {
+    int32_t i;
+    for (i = 0; i < sub->op_args->length; i++) {
+      SPVM_OP* op_arg = SPVM_DYNAMIC_ARRAY_fetch(sub->op_args, i);
+      SPVM_TYPE* arg_type = SPVM_OP_get_type(compiler, op_arg);
+      SPVM_CONSTANT_POOL_push_int(compiler, constant_pool, arg_type->code);
+    }
+  }
+
   // Arg type ids
   constant_pool_sub.arg_type_ids_base = constant_pool->length;
   {
@@ -211,9 +235,6 @@ int32_t SPVM_CONSTANT_POOL_push_sub(SPVM_COMPILER* compiler, SPVM_CONSTANT_POOL*
     }
     constant_pool_sub.object_args_length = object_args_length;
   }
-  if (object_args_length == 0) {
-    constant_pool_sub.object_args_base = -1;
-  }
 
   // Object my_vars length
   int32_t object_my_vars_length = 0;
@@ -232,33 +253,32 @@ int32_t SPVM_CONSTANT_POOL_push_sub(SPVM_COMPILER* compiler, SPVM_CONSTANT_POOL*
     constant_pool_sub.object_my_vars_length = object_my_vars_length;
   }
   
-  if (object_my_vars_length == 0) {
-    constant_pool_sub.object_my_vars_base = -1;
-  }
-
   // Push sub name to constant pool
   constant_pool_sub.abs_name_id = SPVM_CONSTANT_POOL_push_string(compiler, constant_pool, sub->abs_name);
   
   // Push file name to constant pool
   constant_pool_sub.file_name_id = SPVM_CONSTANT_POOL_push_string(compiler, constant_pool, sub->file_name);
   
-  memcpy(&constant_pool->values[start_index], &constant_pool_sub, sizeof(SPVM_CONSTANT_POOL_SUB));
+  memcpy(&constant_pool->values[id], &constant_pool_sub, sizeof(SPVM_CONSTANT_POOL_SUB));
   
-  return start_index;
+  return id;
 }
 
-int32_t SPVM_CONSTANT_POOL_push_field(SPVM_COMPILER* compiler, SPVM_CONSTANT_POOL* constant_pool, SPVM_FIELD* field) {
+int32_t SPVM_CONSTANT_POOL_push_field(SPVM_COMPILER* compiler, SPVM_CONSTANT_POOL* constant_pool, SPVM_FIELD_INFO* field) {
   (void)compiler;
 
-  int32_t start_index = constant_pool->length;
+  int32_t id = constant_pool->length;
 
   // Extend
-  int32_t extend_length = SPVM_CONSTANT_POOL_calculate_extend_length(compiler, constant_pool, sizeof(SPVM_CONSTANT_POOL_FIELD));
+  int32_t extend_length = SPVM_CONSTANT_POOL_calculate_extend_length(compiler, constant_pool, sizeof(SPVM_CONSTANT_POOL_FIELD_INFO));
   SPVM_CONSTANT_POOL_extend(compiler, constant_pool, extend_length);
   
   // Constant pool field information
-  SPVM_CONSTANT_POOL_FIELD constant_pool_field;
-  constant_pool_field.id = field->index;
+  // Field id is field index + 1 because 0 mean no id
+  SPVM_CONSTANT_POOL_FIELD_INFO constant_pool_field;
+  memset(&constant_pool_field, 0, sizeof(SPVM_CONSTANT_POOL_FIELD_INFO));
+  constant_pool_field.index = field->index;
+  constant_pool_field.type_id = field->op_type->uv.type->id;
   
   // Add length
   constant_pool->length += extend_length;
@@ -269,15 +289,15 @@ int32_t SPVM_CONSTANT_POOL_push_field(SPVM_COMPILER* compiler, SPVM_CONSTANT_POO
   // Add field name to constant pool
   constant_pool_field.name_id = SPVM_CONSTANT_POOL_push_string(compiler, constant_pool, field->op_name->uv.name);
   
-  memcpy(&constant_pool->values[start_index], &constant_pool_field, sizeof(SPVM_CONSTANT_POOL_FIELD));
+  memcpy(&constant_pool->values[id], &constant_pool_field, sizeof(SPVM_CONSTANT_POOL_FIELD_INFO));
   
-  return start_index;
+  return id;
 }
 
 int32_t SPVM_CONSTANT_POOL_push_int(SPVM_COMPILER* compiler, SPVM_CONSTANT_POOL* constant_pool, int32_t value) {
   (void)compiler;
 
-  int32_t start_index = constant_pool->length;
+  int32_t id = constant_pool->length;
 
   // Add int value
   int32_t extend_length = SPVM_CONSTANT_POOL_calculate_extend_length(compiler, constant_pool, sizeof(int32_t));
@@ -285,7 +305,7 @@ int32_t SPVM_CONSTANT_POOL_push_int(SPVM_COMPILER* compiler, SPVM_CONSTANT_POOL*
   memcpy(&constant_pool->values[constant_pool->length], &value, sizeof(int32_t));
   constant_pool->length += extend_length;
   
-  return start_index;
+  return id;
 }
 
 int32_t SPVM_CONSTANT_POOL_push_long(SPVM_COMPILER* compiler, SPVM_CONSTANT_POOL* constant_pool, int64_t value) {
@@ -293,7 +313,7 @@ int32_t SPVM_CONSTANT_POOL_push_long(SPVM_COMPILER* compiler, SPVM_CONSTANT_POOL
   
   SPVM_CONSTANT_POOL_adjust_alignment(compiler, constant_pool, sizeof(int64_t));
   
-  int32_t start_index = constant_pool->length;
+  int32_t id = constant_pool->length;
 
   // Add long value
   int32_t extend_length = SPVM_CONSTANT_POOL_calculate_extend_length(compiler, constant_pool, sizeof(int64_t));
@@ -302,13 +322,13 @@ int32_t SPVM_CONSTANT_POOL_push_long(SPVM_COMPILER* compiler, SPVM_CONSTANT_POOL
   
   constant_pool->length += extend_length;
   
-  return start_index;
+  return id;
 }
 
 int32_t SPVM_CONSTANT_POOL_push_float(SPVM_COMPILER* compiler, SPVM_CONSTANT_POOL* constant_pool, float value) {
   (void)compiler;
   
-  int32_t start_index = constant_pool->length;
+  int32_t id = constant_pool->length;
 
   // Add float value
   int32_t extend_length = SPVM_CONSTANT_POOL_calculate_extend_length(compiler, constant_pool, sizeof(float));
@@ -316,7 +336,7 @@ int32_t SPVM_CONSTANT_POOL_push_float(SPVM_COMPILER* compiler, SPVM_CONSTANT_POO
   memcpy(&constant_pool->values[constant_pool->length], &value, sizeof(float));
   constant_pool->length += extend_length;
   
-  return start_index;
+  return id;
 }
 
 int32_t SPVM_CONSTANT_POOL_push_double(SPVM_COMPILER* compiler, SPVM_CONSTANT_POOL* constant_pool, double value) {
@@ -324,7 +344,7 @@ int32_t SPVM_CONSTANT_POOL_push_double(SPVM_COMPILER* compiler, SPVM_CONSTANT_PO
 
   SPVM_CONSTANT_POOL_adjust_alignment(compiler, constant_pool, sizeof(double));
 
-  int32_t start_index = constant_pool->length;
+  int32_t id = constant_pool->length;
 
   // Add double value
   int32_t extend_length = SPVM_CONSTANT_POOL_calculate_extend_length(compiler, constant_pool, sizeof(double));
@@ -332,12 +352,12 @@ int32_t SPVM_CONSTANT_POOL_push_double(SPVM_COMPILER* compiler, SPVM_CONSTANT_PO
   memcpy(&constant_pool->values[constant_pool->length], &value, sizeof(double));
   constant_pool->length += extend_length;
   
-  return start_index;
+  return id;
 }
 
 int32_t SPVM_CONSTANT_POOL_push_string(SPVM_COMPILER* compiler, SPVM_CONSTANT_POOL* constant_pool, const char* string) {
 
-  int32_t start_index = constant_pool->length;
+  int32_t id = constant_pool->length;
 
   // Add string length
   int32_t string_length = (int32_t)strlen(string);
@@ -352,7 +372,7 @@ int32_t SPVM_CONSTANT_POOL_push_string(SPVM_COMPILER* compiler, SPVM_CONSTANT_PO
   
   constant_pool->length += extend_length;
   
-  return start_index;
+  return id;
 }
 
 void SPVM_CONSTANT_POOL_free(SPVM_COMPILER* compiler, SPVM_CONSTANT_POOL* constant_pool) {
