@@ -8,8 +8,9 @@
 #include "spvm_value.h"
 #include "spvm_runtime_api.h"
 #include "spvm_constant_pool_sub.h"
-#include "spvm_constant_pool_field.h"
+#include "spvm_constant_pool_field_info.h"
 #include "spvm_constant_pool_package.h"
+#include "spvm_constant_pool_type.h"
 #include "spvm_object.h"
 #include "spvm_value.h"
 #include "spvm_runtime.h"
@@ -17,10 +18,23 @@
 #include "spvm_api.h"
 #include "spvm_global.h"
 #include "spvm_type.h"
+#include "spvm_hash.h"
+
+SPVM_RUNTIME* SPVM_RUNTIME_API_get_runtime(SPVM_API* api) {
+  (void)api;
+  
+  return SPVM_GLOBAL_RUNTIME;
+}
+
+void SPVM_RUNTIME_API_set_runtime(SPVM_API* api, SPVM_RUNTIME* runtime) {
+  (void)api;
+  
+  SPVM_GLOBAL_RUNTIME = runtime;
+}
 
 int32_t SPVM_RUNTIME_API_get_objects_count(SPVM_API* api) {
   (void)api;
-  return SPVM_GLOBAL_RUNTIME->objects_count;
+  return SPVM_RUNTIME_API_get_runtime(api)->objects_count;
 }
 
 void SPVM_RUNTIME_API_free_weaken_back_refs(SPVM_API* api, SPVM_OBJECT* weaken_back_refs, int32_t weaken_back_refs_length) {
@@ -67,7 +81,7 @@ void SPVM_RUNTIME_API_weaken(SPVM_API* api, SPVM_OBJECT** object_address) {
   
   // Create array of weaken_back_refs if need
   if (object->weaken_back_refs == NULL) {
-    object->weaken_back_refs = SPVM_RUNTIME_API_new_object_array(api, SPVM_TYPE_C_ID_VOID, 1);
+    object->weaken_back_refs = SPVM_RUNTIME_API_new_object_array(api, SPVM_TYPE_C_CODE_VOID, 1);
     object->weaken_back_refs->ref_count++;
   }
   
@@ -79,7 +93,7 @@ void SPVM_RUNTIME_API_weaken(SPVM_API* api, SPVM_OBJECT** object_address) {
   if (length == capacity) {
     
     int32_t new_capacity = capacity * 2;
-    SPVM_OBJECT* new_weaken_back_refs = SPVM_RUNTIME_API_new_object_array(api, SPVM_TYPE_C_ID_VOID, new_capacity);
+    SPVM_OBJECT* new_weaken_back_refs = SPVM_RUNTIME_API_new_object_array(api, SPVM_TYPE_C_CODE_VOID, new_capacity);
     new_weaken_back_refs->ref_count++;
     
     SPVM_OBJECT*** weaken_back_refs_elements = (SPVM_OBJECT***)((intptr_t)object->weaken_back_refs + sizeof(SPVM_OBJECT));
@@ -154,7 +168,7 @@ void SPVM_RUNTIME_API_unweaken(SPVM_API* api, SPVM_OBJECT** object_address) {
 }
 
 void SPVM_RUNTIME_API_set_exception(SPVM_API* api, SPVM_OBJECT* exception) {
-  SPVM_RUNTIME* runtime = SPVM_GLOBAL_RUNTIME;
+  SPVM_RUNTIME* runtime = SPVM_RUNTIME_API_get_runtime(api);
   
   if (runtime->exception != NULL) {
     SPVM_RUNTIME_API_dec_ref_count(api, (SPVM_OBJECT*)runtime->exception);
@@ -170,13 +184,13 @@ void SPVM_RUNTIME_API_set_exception(SPVM_API* api, SPVM_OBJECT* exception) {
 SPVM_OBJECT* SPVM_RUNTIME_API_get_exception(SPVM_API* api) {
   (void)api;
   
-  SPVM_RUNTIME* runtime = SPVM_GLOBAL_RUNTIME;
+  SPVM_RUNTIME* runtime = SPVM_RUNTIME_API_get_runtime(api);
   
   return runtime->exception;
 }
 
 SPVM_OBJECT* SPVM_RUNTIME_API_new_byte_array(SPVM_API* api, int32_t length) {
-  SPVM_RUNTIME* runtime = SPVM_GLOBAL_RUNTIME;
+  SPVM_RUNTIME* runtime = SPVM_RUNTIME_API_get_runtime(api);
   SPVM_RUNTIME_ALLOCATOR* allocator = runtime->allocator;
   
   // Allocate array
@@ -195,10 +209,11 @@ SPVM_OBJECT* SPVM_RUNTIME_API_new_byte_array(SPVM_API* api, int32_t length) {
     ((int8_t*)((intptr_t)object + sizeof(SPVM_OBJECT)))[length] = 0;
     
     // Array
-    object->is_array = 1;
+    object->dimension = 1;
     
     // Set type id
-    object->type_id = SPVM_TYPE_C_ID_BYTE;
+    int32_t* type_code_to_id = (int32_t*)&runtime->constant_pool[runtime->type_code_to_id_base];
+    object->type_id = type_code_to_id[SPVM_TYPE_C_CODE_BYTE_ARRAY];
     
     // Set value type
     object->value_type = SPVM_OBJECT_C_VALUE_TYPE_BYTE;
@@ -214,7 +229,7 @@ SPVM_OBJECT* SPVM_RUNTIME_API_new_byte_array(SPVM_API* api, int32_t length) {
 
 
 SPVM_OBJECT* SPVM_RUNTIME_API_new_short_array(SPVM_API* api, int32_t length) {
-  SPVM_RUNTIME* runtime = SPVM_GLOBAL_RUNTIME;
+  SPVM_RUNTIME* runtime = SPVM_RUNTIME_API_get_runtime(api);
   SPVM_RUNTIME_ALLOCATOR* allocator = runtime->allocator;
   
   // Allocate array
@@ -234,10 +249,11 @@ SPVM_OBJECT* SPVM_RUNTIME_API_new_short_array(SPVM_API* api, int32_t length) {
     ((int16_t*)((intptr_t)object + sizeof(SPVM_OBJECT)))[length] = 0;
     
     // Array
-    object->is_array = 1;
+    object->dimension = 1;
     
     // Set type id
-    object->type_id = SPVM_TYPE_C_ID_SHORT;
+    int32_t* type_code_to_id = (int32_t*)&runtime->constant_pool[runtime->type_code_to_id_base];
+    object->type_id = type_code_to_id[SPVM_TYPE_C_CODE_SHORT_ARRAY];
     
     // Set value type
     object->value_type = SPVM_OBJECT_C_VALUE_TYPE_SHORT;
@@ -252,7 +268,7 @@ SPVM_OBJECT* SPVM_RUNTIME_API_new_short_array(SPVM_API* api, int32_t length) {
 }
 
 SPVM_OBJECT* SPVM_RUNTIME_API_new_int_array(SPVM_API* api, int32_t length) {
-  SPVM_RUNTIME* runtime = SPVM_GLOBAL_RUNTIME;
+  SPVM_RUNTIME* runtime = SPVM_RUNTIME_API_get_runtime(api);
   SPVM_RUNTIME_ALLOCATOR* allocator = runtime->allocator;
   
   // Allocate array
@@ -272,10 +288,11 @@ SPVM_OBJECT* SPVM_RUNTIME_API_new_int_array(SPVM_API* api, int32_t length) {
     ((int32_t*)((intptr_t)object + sizeof(SPVM_OBJECT)))[length] = 0;
     
     // Array
-    object->is_array = 1;
+    object->dimension = 1;
     
     // Set type id
-    object->type_id = SPVM_TYPE_C_ID_INT;
+    int32_t* type_code_to_id = (int32_t*)&runtime->constant_pool[runtime->type_code_to_id_base];
+    object->type_id = type_code_to_id[SPVM_TYPE_C_CODE_INT_ARRAY];
     
     // Set value type
     object->value_type = SPVM_OBJECT_C_VALUE_TYPE_INT;
@@ -290,7 +307,7 @@ SPVM_OBJECT* SPVM_RUNTIME_API_new_int_array(SPVM_API* api, int32_t length) {
 }
 
 SPVM_OBJECT* SPVM_RUNTIME_API_new_long_array(SPVM_API* api, int32_t length) {
-  SPVM_RUNTIME* runtime = SPVM_GLOBAL_RUNTIME;
+  SPVM_RUNTIME* runtime = SPVM_RUNTIME_API_get_runtime(api);
   SPVM_RUNTIME_ALLOCATOR* allocator = runtime->allocator;
   
   // Allocate array
@@ -310,10 +327,11 @@ SPVM_OBJECT* SPVM_RUNTIME_API_new_long_array(SPVM_API* api, int32_t length) {
     ((int64_t*)((intptr_t)object + sizeof(SPVM_OBJECT)))[length] = 0;
     
     // Array
-    object->is_array = 1;
+    object->dimension = 1;
     
     // Set type id
-    object->type_id = SPVM_TYPE_C_ID_LONG;
+    int32_t* type_code_to_id = (int32_t*)&runtime->constant_pool[runtime->type_code_to_id_base];
+    object->type_id = type_code_to_id[SPVM_TYPE_C_CODE_LONG_ARRAY];
     
     // Set value type
     object->value_type = SPVM_OBJECT_C_VALUE_TYPE_LONG;
@@ -328,7 +346,7 @@ SPVM_OBJECT* SPVM_RUNTIME_API_new_long_array(SPVM_API* api, int32_t length) {
 }
 
 SPVM_OBJECT* SPVM_RUNTIME_API_new_float_array(SPVM_API* api, int32_t length) {
-  SPVM_RUNTIME* runtime = SPVM_GLOBAL_RUNTIME;
+  SPVM_RUNTIME* runtime = SPVM_RUNTIME_API_get_runtime(api);
   SPVM_RUNTIME_ALLOCATOR* allocator = runtime->allocator;
   
   // Allocate array
@@ -347,10 +365,11 @@ SPVM_OBJECT* SPVM_RUNTIME_API_new_float_array(SPVM_API* api, int32_t length) {
     ((float*)((intptr_t)object + sizeof(SPVM_OBJECT)))[length] = 0;
     
     // Array
-    object->is_array = 1;
+    object->dimension = 1;
     
     // Set type id
-    object->type_id = SPVM_TYPE_C_ID_FLOAT;
+    int32_t* type_code_to_id = (int32_t*)&runtime->constant_pool[runtime->type_code_to_id_base];
+    object->type_id = type_code_to_id[SPVM_TYPE_C_CODE_FLOAT_ARRAY];
     
     // Set value type
     object->value_type = SPVM_OBJECT_C_VALUE_TYPE_FLOAT;
@@ -365,7 +384,7 @@ SPVM_OBJECT* SPVM_RUNTIME_API_new_float_array(SPVM_API* api, int32_t length) {
 }
 
 SPVM_OBJECT* SPVM_RUNTIME_API_new_double_array(SPVM_API* api, int32_t length) {
-  SPVM_RUNTIME* runtime = SPVM_GLOBAL_RUNTIME;
+  SPVM_RUNTIME* runtime = SPVM_RUNTIME_API_get_runtime(api);
   SPVM_RUNTIME_ALLOCATOR* allocator = runtime->allocator;
   
   // Allocate array
@@ -384,10 +403,11 @@ SPVM_OBJECT* SPVM_RUNTIME_API_new_double_array(SPVM_API* api, int32_t length) {
     ((double*)((intptr_t)object + sizeof(SPVM_OBJECT)))[length] = 0;
     
     // Array
-    object->is_array = 1;
+    object->dimension = 1;
     
     // Set type id
-    object->type_id = SPVM_TYPE_C_ID_DOUBLE;
+    int32_t* type_code_to_id = (int32_t*)&runtime->constant_pool[runtime->type_code_to_id_base];
+    object->type_id = type_code_to_id[SPVM_TYPE_C_CODE_DOUBLE_ARRAY];
     
     // Set value type
     object->value_type = SPVM_OBJECT_C_VALUE_TYPE_DOUBLE;
@@ -402,9 +422,8 @@ SPVM_OBJECT* SPVM_RUNTIME_API_new_double_array(SPVM_API* api, int32_t length) {
 }
 
 SPVM_OBJECT* SPVM_RUNTIME_API_new_object_array(SPVM_API* api, int32_t element_type_id, int32_t length) {
-  (void)element_type_id;
   
-  SPVM_RUNTIME* runtime = SPVM_GLOBAL_RUNTIME;
+  SPVM_RUNTIME* runtime = SPVM_RUNTIME_API_get_runtime(api);
   SPVM_RUNTIME_ALLOCATOR* allocator = runtime->allocator;
   
   // Allocate array
@@ -425,8 +444,13 @@ SPVM_OBJECT* SPVM_RUNTIME_API_new_object_array(SPVM_API* api, int32_t element_ty
     // Initialize by null
     memset(object, 0, array_byte_size);
     
+    // Type id
+    SPVM_CONSTANT_POOL_TYPE* element_type = (SPVM_CONSTANT_POOL_TYPE*)&runtime->constant_pool[element_type_id];
+    int32_t type_id = element_type->parent_type_id;
+    object->type_id = type_id;
+    
     // Set type
-    object->is_array = 1;
+    object->dimension = 1;
     
     // Set value type
     object->value_type = SPVM_OBJECT_C_VALUE_TYPE_OBJECT;
@@ -440,12 +464,16 @@ SPVM_OBJECT* SPVM_RUNTIME_API_new_object_array(SPVM_API* api, int32_t element_ty
   }
 }
 
-SPVM_OBJECT* SPVM_RUNTIME_API_new_object(SPVM_API* api, int32_t package_id) {
+SPVM_OBJECT* SPVM_RUNTIME_API_new_object(SPVM_API* api, int32_t type_id) {
+  
+  SPVM_RUNTIME* runtime = SPVM_RUNTIME_API_get_runtime(api);
   
-  SPVM_RUNTIME* runtime = SPVM_GLOBAL_RUNTIME;
   SPVM_RUNTIME_ALLOCATOR* allocator = runtime->allocator;
+  
   int32_t* constant_pool = runtime->constant_pool;
   
+  SPVM_CONSTANT_POOL_TYPE* constant_pool_type = (SPVM_CONSTANT_POOL_TYPE*)&constant_pool[type_id];
+  int32_t package_id = constant_pool_type->package_id;
   SPVM_CONSTANT_POOL_PACKAGE* constant_pool_package = (SPVM_CONSTANT_POOL_PACKAGE*)&constant_pool[package_id];
   
   // Allocate memory
@@ -453,10 +481,8 @@ SPVM_OBJECT* SPVM_RUNTIME_API_new_object(SPVM_API* api, int32_t package_id) {
   int64_t object_byte_size = (int64_t)sizeof(SPVM_OBJECT) + (int64_t)sizeof(SPVM_VALUE) * (int64_t)length;
   SPVM_OBJECT* object = SPVM_RUNTIME_ALLOCATOR_malloc_zero(api, allocator, object_byte_size);
   
-  // Package constant pool index
-  object->package_id = package_id;
-  
-  object->type_id = constant_pool_package->type_id;
+  // Set type id
+  object->type_id = type_id;
   
   assert(object_byte_size == SPVM_RUNTIME_API_calcurate_object_byte_size(api, object));
   
@@ -545,7 +571,7 @@ void SPVM_RUNTIME_API_inc_dec_ref_count(SPVM_API* api, SPVM_OBJECT* object) {
 }
 
 void SPVM_RUNTIME_API_dec_ref_count(SPVM_API* api, SPVM_OBJECT* object) {
-  SPVM_RUNTIME* runtime = SPVM_GLOBAL_RUNTIME;
+  SPVM_RUNTIME* runtime = SPVM_RUNTIME_API_get_runtime(api);
   
   assert(object != NULL);
   assert(object->ref_count > 0);
@@ -561,7 +587,7 @@ void SPVM_RUNTIME_API_dec_ref_count(SPVM_API* api, SPVM_OBJECT* object) {
   // If reference count is zero, free address.
   if (object->ref_count == 0) {
     // Array
-    if (object->is_array) {
+    if (object->dimension > 0) {
       if (object->value_type == SPVM_OBJECT_C_VALUE_TYPE_OBJECT) {
         
         // Array length
@@ -582,9 +608,8 @@ void SPVM_RUNTIME_API_dec_ref_count(SPVM_API* api, SPVM_OBJECT* object) {
     // Object
     else {
       int32_t* constant_pool = runtime->constant_pool;
-      int32_t package_id = object->package_id;
-      (void)package_id;
-      SPVM_CONSTANT_POOL_PACKAGE* constant_pool_package = (SPVM_CONSTANT_POOL_PACKAGE*)&constant_pool[object->package_id];
+      SPVM_CONSTANT_POOL_TYPE* constant_pool_type = (SPVM_CONSTANT_POOL_TYPE*)&runtime->constant_pool[object->type_id];
+      SPVM_CONSTANT_POOL_PACKAGE* constant_pool_package = (SPVM_CONSTANT_POOL_PACKAGE*)&constant_pool[constant_pool_type->package_id];
       
       int32_t object_fields_length = constant_pool_package->object_fields_length;
       
@@ -630,7 +655,7 @@ int32_t SPVM_RUNTIME_API_get_ref_count(SPVM_API* api, SPVM_OBJECT* object) {
 }
 
 void SPVM_RUNTIME_API_call_void_sub(SPVM_API* api, int32_t sub_id, SPVM_VALUE* args) {
-  SPVM_RUNTIME* runtime = SPVM_GLOBAL_RUNTIME;
+  SPVM_RUNTIME* runtime = SPVM_RUNTIME_API_get_runtime(api);
   
   SPVM_CONSTANT_POOL_SUB* constant_pool_sub = (SPVM_CONSTANT_POOL_SUB*)&runtime->constant_pool[sub_id];
   
@@ -648,7 +673,7 @@ void SPVM_RUNTIME_API_call_void_sub(SPVM_API* api, int32_t sub_id, SPVM_VALUE* a
 }
 
 int8_t SPVM_RUNTIME_API_call_byte_sub(SPVM_API* api, int32_t sub_id, SPVM_VALUE* args) {
-  SPVM_RUNTIME* runtime = SPVM_GLOBAL_RUNTIME;
+  SPVM_RUNTIME* runtime = SPVM_RUNTIME_API_get_runtime(api);
   
   SPVM_CONSTANT_POOL_SUB* constant_pool_sub = (SPVM_CONSTANT_POOL_SUB*)&runtime->constant_pool[sub_id];
   
@@ -675,7 +700,7 @@ int8_t SPVM_RUNTIME_API_call_byte_sub(SPVM_API* api, int32_t sub_id, SPVM_VALUE*
 }
 
 int16_t SPVM_RUNTIME_API_call_short_sub(SPVM_API* api, int32_t sub_id, SPVM_VALUE* args) {
-  SPVM_RUNTIME* runtime = SPVM_GLOBAL_RUNTIME;
+  SPVM_RUNTIME* runtime = SPVM_RUNTIME_API_get_runtime(api);
   
   SPVM_CONSTANT_POOL_SUB* constant_pool_sub = (SPVM_CONSTANT_POOL_SUB*)&runtime->constant_pool[sub_id];
   
@@ -702,7 +727,7 @@ int16_t SPVM_RUNTIME_API_call_short_sub(SPVM_API* api, int32_t sub_id, SPVM_VALU
 }
 
 int32_t SPVM_RUNTIME_API_call_int_sub(SPVM_API* api, int32_t sub_id, SPVM_VALUE* args) {
-  SPVM_RUNTIME* runtime = SPVM_GLOBAL_RUNTIME;
+  SPVM_RUNTIME* runtime = SPVM_RUNTIME_API_get_runtime(api);
   
   SPVM_CONSTANT_POOL_SUB* constant_pool_sub = (SPVM_CONSTANT_POOL_SUB*)&runtime->constant_pool[sub_id];
   
@@ -729,7 +754,7 @@ int32_t SPVM_RUNTIME_API_call_int_sub(SPVM_API* api, int32_t sub_id, SPVM_VALUE*
 }
 
 int64_t SPVM_RUNTIME_API_call_long_sub(SPVM_API* api, int32_t sub_id, SPVM_VALUE* args) {
-  SPVM_RUNTIME* runtime = SPVM_GLOBAL_RUNTIME;
+  SPVM_RUNTIME* runtime = SPVM_RUNTIME_API_get_runtime(api);
   
   SPVM_CONSTANT_POOL_SUB* constant_pool_sub = (SPVM_CONSTANT_POOL_SUB*)&runtime->constant_pool[sub_id];
   
@@ -756,7 +781,7 @@ int64_t SPVM_RUNTIME_API_call_long_sub(SPVM_API* api, int32_t sub_id, SPVM_VALUE
 }
 
 float SPVM_RUNTIME_API_call_float_sub(SPVM_API* api, int32_t sub_id, SPVM_VALUE* args) {
-  SPVM_RUNTIME* runtime = SPVM_GLOBAL_RUNTIME;
+  SPVM_RUNTIME* runtime = SPVM_RUNTIME_API_get_runtime(api);
   
   SPVM_CONSTANT_POOL_SUB* constant_pool_sub = (SPVM_CONSTANT_POOL_SUB*)&runtime->constant_pool[sub_id];
   
@@ -783,7 +808,7 @@ float SPVM_RUNTIME_API_call_float_sub(SPVM_API* api, int32_t sub_id, SPVM_VALUE*
 }
 
 double SPVM_RUNTIME_API_call_double_sub(SPVM_API* api, int32_t sub_id, SPVM_VALUE* args) {
-  SPVM_RUNTIME* runtime = SPVM_GLOBAL_RUNTIME;
+  SPVM_RUNTIME* runtime = SPVM_RUNTIME_API_get_runtime(api);
   
   SPVM_CONSTANT_POOL_SUB* constant_pool_sub = (SPVM_CONSTANT_POOL_SUB*)&runtime->constant_pool[sub_id];
   
@@ -810,7 +835,7 @@ double SPVM_RUNTIME_API_call_double_sub(SPVM_API* api, int32_t sub_id, SPVM_VALU
 }
 
 SPVM_OBJECT* SPVM_RUNTIME_API_call_object_sub(SPVM_API* api, int32_t sub_id, SPVM_VALUE* args) {
-  SPVM_RUNTIME* runtime = SPVM_GLOBAL_RUNTIME;
+  SPVM_RUNTIME* runtime = SPVM_RUNTIME_API_get_runtime(api);
   
   SPVM_CONSTANT_POOL_SUB* constant_pool_sub = (SPVM_CONSTANT_POOL_SUB*)&runtime->constant_pool[sub_id];
   
@@ -839,206 +864,146 @@ SPVM_OBJECT* SPVM_RUNTIME_API_call_object_sub(SPVM_API* api, int32_t sub_id, SPV
 int32_t SPVM_RUNTIME_API_get_field_id(SPVM_API* api, SPVM_OBJECT* object, const char* name) {
   (void)api;
   
-  SPVM_RUNTIME* runtime = SPVM_GLOBAL_RUNTIME;
+  // Runtime
+  SPVM_RUNTIME* runtime = SPVM_RUNTIME_API_get_runtime(api);
   
+  // Constant pool
   int32_t* constant_pool = runtime->constant_pool;
-  SPVM_CONSTANT_POOL_PACKAGE* constant_pool_package = (SPVM_CONSTANT_POOL_PACKAGE*)&constant_pool[object->package_id];
   
-  int32_t length = constant_pool_package->fields_length;
+  // Type
+  SPVM_CONSTANT_POOL_TYPE* constant_pool_type = (SPVM_CONSTANT_POOL_TYPE*)&runtime->constant_pool[object->type_id];
   
-  int32_t fields_base = constant_pool_package->fields_base;
+  // Type name
+  int32_t constant_pool_type_name_id = constant_pool_type->name_id;
+  const char* type_name = (char*)&constant_pool[constant_pool_type_name_id + 1];
   
-  int32_t field_index = SPVM_API_ERROR_NO_ID;
-  _Bool found = 0;
-  {
-    int32_t i;
-    for (i = 0; i < length; i++) {
-      
-      int32_t field_id = constant_pool[fields_base + i];
-      SPVM_CONSTANT_POOL_FIELD* constant_pool_field = (SPVM_CONSTANT_POOL_FIELD*)&constant_pool[field_id];
-      
-      int32_t field_name_id = constant_pool_field->name_id;
-      
-      char* match_name = (char*)&constant_pool[field_name_id + 1];
-      
-      if (strcmp(name, match_name) == 0) {
-        found = 1;
-        field_index = i;
-        break;
-      }
-    }
-  }
+  // Constant pool field symbol table
+  SPVM_HASH* field_id_symtable = runtime->field_id_symtable;
+  SPVM_HASH* field_info_symtable = SPVM_HASH_search(field_id_symtable, type_name, strlen(type_name));
   
-  return field_index;
+  int32_t field_id = (int32_t)(intptr_t)SPVM_HASH_search(field_info_symtable, name, strlen(name));
+  
+  return field_id;
 }
 
 int32_t SPVM_RUNTIME_API_get_sub_id(SPVM_API* api, const char* name) {
   (void)api;
   
-  SPVM_RUNTIME* runtime = SPVM_GLOBAL_RUNTIME;
-  
-  int32_t* constant_pool = runtime->constant_pool;
-  int32_t length = runtime->subs_length;
-  int32_t subs_base = runtime->subs_base;
-  
-  int32_t found_sub_id = SPVM_API_ERROR_NO_ID;
-  _Bool found = 0;
-  {
-    int32_t i;
-    for (i = 0; i < length; i++) {
-      int32_t sub_id = constant_pool[subs_base + i];
-      SPVM_CONSTANT_POOL_SUB* constant_pool_sub = (SPVM_CONSTANT_POOL_SUB*)&runtime->constant_pool[sub_id];
-      
-      int32_t sub_name_id = constant_pool_sub->abs_name_id;
-      
-      char* match_name = (char*)&constant_pool[sub_name_id + 1];
-      if (strcmp(name, match_name) == 0) {
-        found = 1;
-        found_sub_id = sub_id;
-        break;
-      }
-    }
+  if (name == NULL) {
+    return 0;
   }
   
-  if (!found) {
-    fprintf(stderr, "Can't find subroutine name \"%s\"\n", name);
-    abort();
-  }
+  SPVM_RUNTIME* runtime = SPVM_RUNTIME_API_get_runtime(api);
+  SPVM_HASH* sub_id_symtable = runtime->sub_id_symtable;
+  int32_t constant_pool_sub_id = (int32_t)(intptr_t)SPVM_HASH_search(sub_id_symtable, name, strlen(name));
   
-  return found_sub_id;
+  return constant_pool_sub_id;
 }
 
-int32_t SPVM_RUNTIME_API_get_package_id(SPVM_API* api, const char* name) {
+int32_t SPVM_RUNTIME_API_get_type_id(SPVM_API* api, const char* name) {
   (void)api;
   
-  SPVM_RUNTIME* runtime = SPVM_GLOBAL_RUNTIME;
-  
-  int32_t* constant_pool = runtime->constant_pool;
-  int32_t length = runtime->packages_length;
-  int32_t packages_base = runtime-> packages_base;
-  
-  int32_t found_package_id = SPVM_API_ERROR_NO_ID;
-  _Bool found = 0;
-  {
-    int32_t i;
-    for (i = 0; i < length; i++) {
-      int32_t package_id = constant_pool[packages_base + i];
-      SPVM_CONSTANT_POOL_PACKAGE* constant_pool_package = (SPVM_CONSTANT_POOL_PACKAGE*)&constant_pool[package_id];
-      
-      int32_t package_name_id = constant_pool_package->name_id;
-      
-      char* match_name = (char*)&constant_pool[package_name_id + 1];
-      if (strcmp(name, match_name) == 0) {
-        found = 1;
-        found_package_id = package_id;
-        break;
-      }
-    }
-  }
-  
-  if (!found) {
-    fprintf(stderr, "Can't find packageroutine name \"%s\"\n", name);
-    abort();
-  }
-  
-  return found_package_id;
+  SPVM_RUNTIME* runtime = SPVM_RUNTIME_API_get_runtime(api);
+  SPVM_HASH* type_id_symtable = runtime->type_id_symtable;
+  int32_t constant_pool_type_id = (int32_t)(intptr_t)SPVM_HASH_search(type_id_symtable, name, strlen(name));
+
+  return constant_pool_type_id;
 }
 
-
-int8_t SPVM_RUNTIME_API_get_byte_field(SPVM_API* api, SPVM_OBJECT* object, int32_t field_index) {
+int8_t SPVM_RUNTIME_API_get_byte_field(SPVM_API* api, SPVM_OBJECT* object, int32_t field_id) {
   SPVM_VALUE* fields = SPVM_RUNTIME_API_get_fields(api, object);
-  int8_t byte_value = fields[field_index].byte_value;
+  int8_t byte_value = fields[field_id - 1].byte_value;
   
   return byte_value;
 }
 
-int16_t SPVM_RUNTIME_API_get_short_field(SPVM_API* api, SPVM_OBJECT* object, int32_t field_index) {
+int16_t SPVM_RUNTIME_API_get_short_field(SPVM_API* api, SPVM_OBJECT* object, int32_t field_id) {
   SPVM_VALUE* fields = SPVM_RUNTIME_API_get_fields(api, object);
-  int16_t short_value = fields[field_index].short_value;
+  int16_t short_value = fields[field_id - 1].short_value;
   
   return short_value;
 }
 
-int32_t SPVM_RUNTIME_API_get_int_field(SPVM_API* api, SPVM_OBJECT* object, int32_t field_index) {
+int32_t SPVM_RUNTIME_API_get_int_field(SPVM_API* api, SPVM_OBJECT* object, int32_t field_id) {
   SPVM_VALUE* fields = SPVM_RUNTIME_API_get_fields(api, object);
-  int32_t int_value = fields[field_index].int_value;
+  int32_t int_value = fields[field_id - 1].int_value;
   
   return int_value;
 }
 
-int64_t SPVM_RUNTIME_API_get_long_field(SPVM_API* api, SPVM_OBJECT* object, int32_t field_index) {
+int64_t SPVM_RUNTIME_API_get_long_field(SPVM_API* api, SPVM_OBJECT* object, int32_t field_id) {
   SPVM_VALUE* fields = SPVM_RUNTIME_API_get_fields(api, object);
-  int64_t long_value = fields[field_index].long_value;
+  int64_t long_value = fields[field_id - 1].long_value;
   
   return long_value;
 }
 
-float SPVM_RUNTIME_API_get_float_field(SPVM_API* api, SPVM_OBJECT* object, int32_t field_index) {
+float SPVM_RUNTIME_API_get_float_field(SPVM_API* api, SPVM_OBJECT* object, int32_t field_id) {
   SPVM_VALUE* fields = SPVM_RUNTIME_API_get_fields(api, object);
-  float float_value = fields[field_index].float_value;
+  float float_value = fields[field_id - 1].float_value;
   
   return float_value;
 }
 
-double SPVM_RUNTIME_API_get_double_field(SPVM_API* api, SPVM_OBJECT* object, int32_t field_index) {
+double SPVM_RUNTIME_API_get_double_field(SPVM_API* api, SPVM_OBJECT* object, int32_t field_id) {
   SPVM_VALUE* fields = SPVM_RUNTIME_API_get_fields(api, object);
-  double double_value = fields[field_index].double_value;
+  double double_value = fields[field_id - 1].double_value;
   
   return double_value;
 }
 
-SPVM_OBJECT* SPVM_RUNTIME_API_get_object_field(SPVM_API* api, SPVM_OBJECT* object, int32_t field_index) {
+SPVM_OBJECT* SPVM_RUNTIME_API_get_object_field(SPVM_API* api, SPVM_OBJECT* object, int32_t field_id) {
   SPVM_VALUE* fields = SPVM_RUNTIME_API_get_fields(api, object);
-  SPVM_OBJECT* object_value = fields[field_index].object_value;
+  SPVM_OBJECT* object_value = fields[field_id - 1].object_value;
   
   return object_value;
 }
 
-void SPVM_RUNTIME_API_set_byte_field(SPVM_API* api, SPVM_OBJECT* object, int32_t field_index, int8_t value) {
+void SPVM_RUNTIME_API_set_byte_field(SPVM_API* api, SPVM_OBJECT* object, int32_t field_id, int8_t value) {
   SPVM_VALUE* fields = SPVM_RUNTIME_API_get_fields(api, object);
-  fields[field_index].byte_value = value;
+  fields[field_id - 1].byte_value = value;
 }
 
-void SPVM_RUNTIME_API_set_short_field(SPVM_API* api, SPVM_OBJECT* object, int32_t field_index, int16_t value) {
+void SPVM_RUNTIME_API_set_short_field(SPVM_API* api, SPVM_OBJECT* object, int32_t field_id, int16_t value) {
   SPVM_VALUE* fields = SPVM_RUNTIME_API_get_fields(api, object);
-  fields[field_index].short_value = value;
+  fields[field_id - 1].short_value = value;
 }
 
-void SPVM_RUNTIME_API_set_int_field(SPVM_API* api, SPVM_OBJECT* object, int32_t field_index, int32_t value) {
+void SPVM_RUNTIME_API_set_int_field(SPVM_API* api, SPVM_OBJECT* object, int32_t field_id, int32_t value) {
   SPVM_VALUE* fields = SPVM_RUNTIME_API_get_fields(api, object);
-  fields[field_index].int_value = value;
+  fields[field_id - 1].int_value = value;
 }
 
-void SPVM_RUNTIME_API_set_long_field(SPVM_API* api, SPVM_OBJECT* object, int32_t field_index, int64_t value) {
+void SPVM_RUNTIME_API_set_long_field(SPVM_API* api, SPVM_OBJECT* object, int32_t field_id, int64_t value) {
   SPVM_VALUE* fields = SPVM_RUNTIME_API_get_fields(api, object);
-  fields[field_index].long_value = value;
+  fields[field_id - 1].long_value = value;
 }
 
-void SPVM_RUNTIME_API_set_float_field(SPVM_API* api, SPVM_OBJECT* object, int32_t field_index, float value) {
+void SPVM_RUNTIME_API_set_float_field(SPVM_API* api, SPVM_OBJECT* object, int32_t field_id, float value) {
   SPVM_VALUE* fields = SPVM_RUNTIME_API_get_fields(api, object);
-  fields[field_index].float_value = value;
+  fields[field_id - 1].float_value = value;
 }
 
-void SPVM_RUNTIME_API_set_double_field(SPVM_API* api, SPVM_OBJECT* object, int32_t field_index, double value) {
+void SPVM_RUNTIME_API_set_double_field(SPVM_API* api, SPVM_OBJECT* object, int32_t field_id, double value) {
   SPVM_VALUE* fields = SPVM_RUNTIME_API_get_fields(api, object);
-  fields[field_index].double_value = value;
+  fields[field_id - 1].double_value = value;
 }
 
-void SPVM_RUNTIME_API_set_object_field(SPVM_API* api, SPVM_OBJECT* object, int32_t field_index, SPVM_OBJECT* value) {
+void SPVM_RUNTIME_API_set_object_field(SPVM_API* api, SPVM_OBJECT* object, int32_t field_id, SPVM_OBJECT* value) {
   SPVM_VALUE* fields = SPVM_RUNTIME_API_get_fields(api, object);
   
-  if(fields[field_index].object_value != NULL) {
+  if(fields[field_id - 1].object_value != NULL) {
     // If object is weak, unweaken
-    if (SPVM_RUNTIME_API_isweak(api, fields[field_index].object_value)) {
-      SPVM_RUNTIME_API_unweaken(api, (SPVM_OBJECT**)&fields[field_index]);
+    if (SPVM_RUNTIME_API_isweak(api, fields[field_id - 1].object_value)) {
+      SPVM_RUNTIME_API_unweaken(api, (SPVM_OBJECT**)&fields[field_id - 1]);
     }
-    api->dec_ref_count(api, fields[field_index].object_value);
+    api->dec_ref_count(api, fields[field_id - 1].object_value);
   }
   
-  fields[field_index].object_value = value;
+  fields[field_id - 1].object_value = value;
   
-  if(fields[field_index].object_value != NULL) {
-    api->inc_ref_count(api, fields[field_index].object_value);
+  if(fields[field_id - 1].object_value != NULL) {
+    api->inc_ref_count(api, fields[field_id - 1].object_value);
   }
 }
 
@@ -1060,17 +1025,19 @@ int32_t SPVM_RUNTIME_API_get_array_value_size(SPVM_API* api, int32_t type) {
 
 int64_t SPVM_RUNTIME_API_calcurate_object_byte_size(SPVM_API* api, SPVM_OBJECT* object) {
   
-  SPVM_RUNTIME* runtime = SPVM_GLOBAL_RUNTIME;
+  SPVM_RUNTIME* runtime = SPVM_RUNTIME_API_get_runtime(api);
   
   int64_t byte_size;
   
   // Reference is string
-  if (object->is_array) {
+  if (object->dimension > 0) {
     byte_size = sizeof(SPVM_OBJECT) + (object->length + 1) * SPVM_RUNTIME_API_get_array_value_size(api, object->value_type);
   }
   // Reference is object
   else {
-    SPVM_CONSTANT_POOL_PACKAGE* constant_pool_package = (SPVM_CONSTANT_POOL_PACKAGE*)&runtime->constant_pool[object->package_id];
+    SPVM_CONSTANT_POOL_TYPE* constant_pool_type = (SPVM_CONSTANT_POOL_TYPE*)&runtime->constant_pool[object->type_id];
+    
+    SPVM_CONSTANT_POOL_PACKAGE* constant_pool_package = (SPVM_CONSTANT_POOL_PACKAGE*)&runtime->constant_pool[constant_pool_type->package_id];
     byte_size = sizeof(SPVM_OBJECT) + sizeof(SPVM_VALUE) * constant_pool_package->fields_length;
   }
   
@@ -1080,9 +1047,15 @@ int64_t SPVM_RUNTIME_API_calcurate_object_byte_size(SPVM_API* api, SPVM_OBJECT*
 SPVM_OBJECT* SPVM_RUNTIME_API_new_string(SPVM_API* api, const char* string) {
   (void)api;
   
+  SPVM_RUNTIME* runtime = SPVM_RUNTIME_API_get_runtime(api);
+  
   int32_t length = strlen(string);
   SPVM_OBJECT* object = SPVM_RUNTIME_API_new_byte_array(api, length);
   
+  // Set type id
+  int32_t* type_code_to_id = (int32_t*)&runtime->constant_pool[runtime->type_code_to_id_base];
+  object->type_id = type_code_to_id[SPVM_TYPE_C_CODE_STRING];
+  
   // Copy string
   memcpy((void*)((intptr_t)object + sizeof(SPVM_OBJECT)), string, length + 1);
   
@@ -1098,10 +1071,12 @@ SPVM_VALUE* SPVM_RUNTIME_API_get_fields(SPVM_API* api, SPVM_OBJECT* object) {
 int32_t SPVM_RUNTIME_API_get_fields_length(SPVM_API* api, SPVM_OBJECT* object) {
   (void)api;
   
-  SPVM_RUNTIME* runtime = SPVM_GLOBAL_RUNTIME;
+  SPVM_RUNTIME* runtime = SPVM_RUNTIME_API_get_runtime(api);
   
+  SPVM_CONSTANT_POOL_TYPE* constant_pool_type = (SPVM_CONSTANT_POOL_TYPE*)&runtime->constant_pool[object->type_id];
+
   int32_t* constant_pool = runtime->constant_pool;
-  SPVM_CONSTANT_POOL_PACKAGE* constant_pool_package = (SPVM_CONSTANT_POOL_PACKAGE*)&constant_pool[object->package_id];
+  SPVM_CONSTANT_POOL_PACKAGE* constant_pool_package = (SPVM_CONSTANT_POOL_PACKAGE*)&constant_pool[constant_pool_type->package_id];
   int32_t length = constant_pool_package->fields_length;
   
   return length;
@@ -1111,10 +1086,12 @@ int32_t SPVM_RUNTIME_API_get_fields_length(SPVM_API* api, SPVM_OBJECT* object) {
 int32_t SPVM_RUNTIME_API_dump_field_names(SPVM_API* api, SPVM_OBJECT* object) {
   (void)api;
   
-  SPVM_RUNTIME* runtime = SPVM_GLOBAL_RUNTIME;
+  SPVM_RUNTIME* runtime = SPVM_RUNTIME_API_get_runtime(api);
+  
+  SPVM_CONSTANT_POOL_TYPE* constant_pool_type = (SPVM_CONSTANT_POOL_TYPE*)&runtime->constant_pool[object->type_id];
   
   int32_t* constant_pool = runtime->constant_pool;
-  SPVM_CONSTANT_POOL_PACKAGE* constant_pool_package = (SPVM_CONSTANT_POOL_PACKAGE*)&constant_pool[object->package_id];
+  SPVM_CONSTANT_POOL_PACKAGE* constant_pool_package = (SPVM_CONSTANT_POOL_PACKAGE*)&constant_pool[constant_pool_type->package_id];
   int32_t field_names_base = constant_pool_package->field_names_base;
   int32_t length = constant_pool_package->fields_length;
   
